"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
const mongoose = require("mongoose");
const utils_1 = require("@nodeswork/utils");
const sMongoose = require("../mongoose");
class MetricsModel extends sMongoose.Model {
    static async upsertMetrics(options) {
        const data = utils_1.metrics.operator.updateMetricsData(options.dimensions, options.name, options.value);
        const updateDoc = {};
        for (const dhash of Object.keys(data.dimensions)) {
            updateDoc[`dimensions.${dhash}`] = data.dimensions[dhash];
        }
        for (const mName of Object.keys(data.metrics)) {
            const mValues = data.metrics[mName];
            for (const dhash of Object.keys(mValues)) {
                updateDoc[`metrics.${mName}.${dhash}`] = mValues[dhash];
            }
        }
        const self = this.cast();
        return await self.findByIdAndUpdate(options.id, { $set: updateDoc }, {
            new: true,
            upsert: true,
        });
    }
    appendMetrics(options) {
        utils_1.metrics.operator.updateMetricsData(options.dimensions, options.name, options.value, this);
        this.markModified('dimensions');
        this.markModified('metrics');
    }
    static async appendMetrics(options) {
        const self = this.cast();
        const metrics = await self.findById(options.id);
        if (metrics == null) {
            return await this.createMetrics({ metrics: [options] });
        }
        else {
            utils_1.metrics.operator.updateMetricsData(options.dimensions, options.name, options.value, metrics);
            return await metrics.save();
        }
    }
    static async createMetrics(options) {
        const self = this.cast();
        const doc = _.extend({}, options.doc, {
            dimensions: {},
            metrics: {},
        });
        if (options.ts) {
            doc.ts = options.ts;
        }
        if (options.metrics && options.metrics.length) {
            for (const mo of options.metrics) {
                utils_1.metrics.operator.updateMetricsData(mo.dimensions, mo.name, mo.value, doc);
            }
        }
        return (await self.create(doc));
    }
    static async searchMetrics(options) {
        const self = this.cast();
        const query = _.extend({}, options.query);
        const project = _.extend({}, options.project);
        const dimensions = _.flatten([options.dimensions || []]);
        const metricsNames = _.flatten([options.metrics || []]);
        query.ts = {
            $gte: options.timerange.start,
            $lt: options.timerange.end,
        };
        project.dimensions = 1;
        if (metricsNames.length > 0) {
            for (const mName of metricsNames) {
                project[`metrics.${mName}`] = 1;
            }
        }
        const result = await self.find(query, project, options.queryOptions);
        for (const v of result) {
            const nd = utils_1.metrics.operator.projectMetricsData(v, {
                dimensions,
                metrics: metricsNames,
            });
            _.extend(v, nd);
        }
        return result;
    }
    static async aggregateMetrics(options) {
        const metricsModels = await this.searchMetrics(options);
        const result = [];
        return _
            .chain(metricsModels)
            .groupBy((model) => {
            return Math.floor(model.ts.getTime() / options.granularityInSecond);
        })
            .map((ms, startTime) => {
            const merged = utils_1.metrics.operator.mergeMetricsData(ms);
            return {
                timerange: {
                    start: new Date(startTime),
                    end: new Date(startTime + options.granularityInSecond),
                },
                dimensions: merged.dimensions,
                metrics: merged.metrics,
            };
        })
            .value();
    }
}
__decorate([
    sMongoose.Field({
        type: mongoose.Schema.Types.Date,
        default: Date.now,
    }),
    __metadata("design:type", Date)
], MetricsModel.prototype, "ts", void 0);
__decorate([
    sMongoose.Field({
        type: mongoose.Schema.Types.Mixed,
        default: {},
    }),
    __metadata("design:type", Object)
], MetricsModel.prototype, "dimensions", void 0);
__decorate([
    sMongoose.Field({
        type: mongoose.Schema.Types.Mixed,
        default: {},
    }),
    __metadata("design:type", Object)
], MetricsModel.prototype, "metrics", void 0);
exports.MetricsModel = MetricsModel;

//# sourceMappingURL=metrics-models.js.map
